#property show_inputs

#define MAX_AMOUNTSYMBOLS 10
#define MAX_POINTS 100000

extern string SymbolsStr = "AUDUSD, NZDUSD, EURUSD, GBPUSD, USDCHF, USDJPY, USDCAD"; // for example: "EURGBP, ..., XAGUSD"
extern string BaseCurrency = "USD";
extern datetime StartTime = D'2010.01.01';
extern datetime EndTime = D'2011.01.01';
extern double Value = 0.002;
extern string FileNameAnalyse = "Analyse.prn";
extern string FileNameCorrelations = "Correlations.txt";

int Points[MAX_AMOUNTSYMBOLS][MAX_POINTS], Amount[MAX_AMOUNTSYMBOLS];
double Values[MAX_AMOUNTSYMBOLS][MAX_POINTS];
string Symbols[MAX_AMOUNTSYMBOLS];
int AmountSymbols;

string StrDelSpaces( string Str )
{
  int Pos, Length;

  Str = StringTrimLeft(Str);
  Str = StringTrimRight(Str);

  Length = StringLen(Str) - 1;
  Pos = 1;

  while (Pos < Length)
    if (StringGetChar(Str, Pos) == ' ')
    {  
      Str = StringSubstr(Str, 0, Pos) + StringSubstr(Str, Pos + 1, 0);
      Length--;
    }
    else 
      Pos++;

  return(Str);
}

int StrToStringS( string Str, string Razdelitel, string &Output[] )
{
  int Pos, LengthSh;
  int Count = 0;

  Str = StrDelSpaces(Str);
  Razdelitel = StrDelSpaces(Razdelitel);

  LengthSh = StringLen(Razdelitel);

  while (TRUE)
  {
    Pos = StringFind(Str, Razdelitel);
    Output[Count] = StringSubstr(Str, 0, Pos);
    Count++;
 
    if (Pos == -1)
      break;
 
    Pos += LengthSh;
    Str = StringSubstr(Str, Pos);
  }

  return(Count);
}

datetime GetStartTime( datetime StartTime )
{
  datetime Tmp;
  int Pos;
  
  for (int i = 0; i < AmountSymbols; i++)
  {
    Pos = iBarShift(Symbols[i], Period(), StartTime);
    Tmp = iTime(Symbols[i], Period(), Pos);
    
    if (Tmp > StartTime)
      StartTime = Tmp;
  }
  
  return(StartTime);
}

void init()
{
  AmountSymbols = StrToStringS(SymbolsStr, ",", Symbols);
  StartTime = GetStartTime(StartTime);
    
  return;
}

void GetPrices( string Symb, int Pos, double& PriceHigh, double& PriceLow )
{
  if (StringSubstr(Symb, 0, 3) == BaseCurrency)
  {
    PriceHigh = 1 / iLow(Symb, Period(), Pos);
    PriceLow = 1 / iHigh(Symb, Period(), Pos);
  }
  else
  {
    PriceHigh = iHigh(Symb, Period(), Pos);
    PriceLow = iLow(Symb, Period(), Pos);
  }
  
  return;
}

void GetZigZagInfo( int NumSymbol, double Value )
{
  bool FlagUP = TRUE;
  double Max, Min;
  double PriceHigh, PriceLow;
  int time;
  string Symb = Symbols[NumSymbol];
  double ValueUp = 1 + Value;
  double ValueDown = 1 - Value;
  int Pos = iBarShift(Symb, Period(), StartTime);

  Amount[NumSymbol] = 0;
  Points[NumSymbol][0] = StartTime;
  
  GetPrices(Symb, Pos, Max, Min);
  
  while (Pos >= 0)
  {
    time = iTime(Symb, Period(), Pos);
    
    if (time > EndTime)
      break;

    GetPrices(Symb, Pos, PriceHigh, PriceLow);
      
    if (FlagUP)
    {
      if (PriceHigh > Max)
      {
        Max = PriceHigh;
        Points[NumSymbol][Amount[NumSymbol]] = time;
      }
      else if (PriceLow / Max <= ValueDown)
      {
        Values[NumSymbol][Amount[NumSymbol]] = Max / Min - 1;
        Amount[NumSymbol]++;
        Points[NumSymbol][Amount[NumSymbol]] = time;

        FlagUP = FALSE;
        Min = PriceLow;
      }
    }
    else // (FlagUP == FALSE)
    {
      if (PriceLow < Min)
      {
        Min = PriceLow;

        Points[NumSymbol][Amount[NumSymbol]] = time;
      }
      else if (PriceHigh / Min >= ValueUp)
      {
        Values[NumSymbol][Amount[NumSymbol]] = Min / Max - 1;
        Amount[NumSymbol]++;
        Points[NumSymbol][Amount[NumSymbol]] = time;
        
        FlagUP = TRUE;
        Max = PriceHigh;
      }
    }
      
    Pos--;
  }
  
  return;
}

int GetNextPos( int Pos[] )
{
  int Min = Points[0][Pos[0] + 1];
  int Res = 0;
  
  for (int i = 1; i < AmountSymbols; i++)
    if (Points[i][Pos[i] + 1] < Min)
    {
      Min = Points[i][Pos[i] + 1];
      Res = i;
    }
    
  return(Res);
}

void SaveData4Analyse( string FileName )
{
  string Str;
  int Pos[MAX_AMOUNTSYMBOLS];
  int i, Next = 0;
  int handle = FileOpen(FileName, FILE_CSV|FILE_WRITE);
  
  for (i = 0; i < AmountSymbols; i++)
    Pos[i] = 0;
    
  while (Pos[Next] < Amount[Next] - 1)
  {
    Str = DoubleToStr(Values[0][Pos[0]], 8);
  
    for (i = 1; i < AmountSymbols; i++)
      Str = Str + " " + DoubleToStr(Values[i][Pos[i]], 8);
      
    FileWrite(handle, Str);

    Next = GetNextPos(Pos);
    Pos[Next]++;
  }
  
  FileClose(handle);
    
  return;
}

void SaveCorrelations( string FileName )
{
  int i, j, i2, j2;
  string Str = "Symbols:";
  int handle = FileOpen(FileName, FILE_CSV|FILE_WRITE);
  
  for (i = 0; i < AmountSymbols; i++)
    Str = Str + " " + Symbols[i] + "_" + i;
    
  FileWrite(handle, Str);
  
  Str = "Interval: " + TimeToStr(StartTime, TIME_DATE) + " - " + TimeToStr(EndTime, TIME_DATE);
  FileWrite(handle, Str);

  Str = "Price MinChange = " + DoubleToStr(Value * 100, 2) + "%\n";
  FileWrite(handle, Str);

  int Count = 0;
    
  for (i = 0; i < AmountSymbols - 1; i++)
    for (j = i + 1; j < AmountSymbols; j++)
    {
      if (Amount[i] > Amount[j])
      {
        i2 = j;
        j2 = i;
      }
      else
      {
        i2 = i;
        j2 = j;
      }

      Str = "Correlation (MinChange = " + DoubleToStr(Value * 100, 2) + "%) " +
            Symbols[i2] + "(" + Amount[i2] + ") / " +
            Symbols[j2] + "(" + Amount[j2] + ") = " + DoubleToStr(1.0 * Amount[i2] / Amount[j2], 3);
      FileWrite(handle, Str);
    }

  return;
}

void start()
{
  for (int i = 0; i < AmountSymbols; i++)
    GetZigZagInfo(i, Value);
 
  SaveData4Analyse(FileNameAnalyse);
  SaveCorrelations(FileNameCorrelations);
    
  return; 
}